package java.nio.channels;

import java.util.concurrent.TimeUnit;
import java.nio.ByteBuffer;
import java.util.concurrent.Future;
import java.net.SocketOption;
import java.net.SocketAddress;
import java.io.IOException;
import java.nio.channels.spi.AsynchronousChannelProvider;

public abstract class AsynchronousSocketChannel implements AsynchronousByteChannel, NetworkChannel
{
    private final AsynchronousChannelProvider provider;
    
    protected AsynchronousSocketChannel(AsynchronousChannelProvider provider) {
        this.provider = provider;
    }
    
    public final AsynchronousChannelProvider provider() {
        return this.provider;
    }
    
    public static AsynchronousSocketChannel open(AsynchronousChannelGroup asynchronousChannelGroup) throws IOException {
        return ((asynchronousChannelGroup == null) ? AsynchronousChannelProvider.provider() : asynchronousChannelGroup.provider()).openAsynchronousSocketChannel(asynchronousChannelGroup);
    }
    
    public static AsynchronousSocketChannel open() throws IOException {
        return open(null);
    }
    
    @Override
    public abstract AsynchronousSocketChannel bind(SocketAddress p0) throws IOException;
    
    @Override
    public abstract <T> AsynchronousSocketChannel setOption(SocketOption<T> p0, T p1) throws IOException;
    
    public abstract AsynchronousSocketChannel shutdownInput() throws IOException;
    
    public abstract AsynchronousSocketChannel shutdownOutput() throws IOException;
    
    public abstract SocketAddress getRemoteAddress() throws IOException;
    
    public abstract <A> void connect(SocketAddress p0, A p1, CompletionHandler<Void, ? super A> p2);
    
    public abstract Future<Void> connect(SocketAddress p0);
    
    public abstract <A> void read(ByteBuffer p0, long p1, TimeUnit p2, A p3, CompletionHandler<Integer, ? super A> p4);
    
    @Override
    public final <A> void read(ByteBuffer byteBuffer, A a, CompletionHandler<Integer, ? super A> completionHandler) {
        this.read(byteBuffer, 0L, TimeUnit.MILLISECONDS, a, completionHandler);
    }
    
    @Override
    public abstract Future<Integer> read(ByteBuffer p0);
    
    public abstract <A> void read(ByteBuffer[] p0, int p1, int p2, long p3, TimeUnit p4, A p5, CompletionHandler<Long, ? super A> p6);
    
    public abstract <A> void write(ByteBuffer p0, long p1, TimeUnit p2, A p3, CompletionHandler<Integer, ? super A> p4);
    
    @Override
    public final <A> void write(ByteBuffer byteBuffer, A a, CompletionHandler<Integer, ? super A> completionHandler) {
        this.write(byteBuffer, 0L, TimeUnit.MILLISECONDS, a, completionHandler);
    }
    
    @Override
    public abstract Future<Integer> write(ByteBuffer p0);
    
    public abstract <A> void write(ByteBuffer[] p0, int p1, int p2, long p3, TimeUnit p4, A p5, CompletionHandler<Long, ? super A> p6);
    
    @Override
    public abstract SocketAddress getLocalAddress() throws IOException;
}
